package disks

import (
	"context"

	"github.com/google/uuid"
	"github.com/hashicorp/terraform-plugin-sdk/v2/diag"
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/constants"
)

func dataSourceDiskListTypesDetailedRead(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	listTypesDetailed, err := utilityDiskListTypesDetailedCheckPresence(ctx, d, m)
	if err != nil {
		return diag.FromErr(err)
	}

	id := uuid.New()
	d.SetId(id.String())
	d.Set("items", flattenDiskListTypesDetailed(listTypesDetailed))
	d.Set("entry_count", listTypesDetailed.EntryCount)
	return nil
}

func DataSourceDiskListTypesDetailed() *schema.Resource {
	return &schema.Resource{
		SchemaVersion: 1,
		ReadContext:   dataSourceDiskListTypesDetailedRead,

		Timeouts: &schema.ResourceTimeout{
			Read:    &constants.Timeout30s,
			Default: &constants.Timeout60s,
		},

		Schema: map[string]*schema.Schema{
			"items": {
				Type:     schema.TypeList,
				Computed: true,
				Elem: &schema.Resource{
					Schema: map[string]*schema.Schema{
						"pools": {
							Type:     schema.TypeList,
							Computed: true,
							Elem: &schema.Resource{
								Schema: map[string]*schema.Schema{
									"name": {
										Type:        schema.TypeString,
										Computed:    true,
										Description: "Pool name",
									},
									"system": {
										Type:     schema.TypeString,
										Computed: true,
									},
									"types": {
										Type:     schema.TypeList,
										Computed: true,
										Elem: &schema.Schema{
											Type: schema.TypeString,
										},
										Description: "The types of disk in terms of its role in compute: 'B=Boot, D=Data, T=Temp'",
									},
								},
							},
						},
						"sep_id": {
							Type:        schema.TypeInt,
							Computed:    true,
							Description: "Storage endpoint provider ID to create disk",
						},
						"sep_name": {
							Type:     schema.TypeString,
							Computed: true,
						},
					},
				},
			},
			"entry_count": {
				Type:     schema.TypeInt,
				Computed: true,
			},
		},
	}
}
