package disks

import (
	"context"
	"fmt"

	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/account"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/grid"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
)

func existAccountID(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	c := m.(*controller.ControllerCfg)
	accountID := uint64(d.Get("account_id").(int))

	accountList, err := c.CloudBroker().Account().List(ctx, account.ListRequest{})
	if err != nil {
		return err
	}

	if len(accountList.FilterByID(accountID).Data) == 0 {
		return fmt.Errorf("resourceDiskCreate: can't create/update Disk because AccountID %d is not allowed or does not exist", accountID)
	}

	return nil
}

func existGID(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	c := m.(*controller.ControllerCfg)

	gid := uint64(d.Get("gid").(int))

	gidList, err := c.CloudBroker().Grid().List(ctx, grid.ListRequest{})
	if err != nil {
		return err
	}

	for _, elem := range gidList.Data {
		if elem.GID == gid {
			return nil
		}
	}

	return fmt.Errorf("resourceDiskCreate: can't create/update Disk because GID %d is not allowed or does not exist", gid)
}

func checkParamsExists(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	err := existAccountID(ctx, d, m)
	if err != nil {
		return err
	}

	return existGID(ctx, d, m)
}
